#include "lamparavisual.h"

LamparaVisual::LamparaVisual()
{	
	/*	El bitmap ya ha sido
		creado en el constructor de la clase padre ElementoVisual,
		por lo que aqui podemos cargarlo sin preocuparnos de reservar */
		
		wxString ficheroIcono ("iconos/Lampara.xpm");
		bitmapEnPantalla.LoadFile (ficheroIcono, TIPO_ICONOS);
		
		wxString ficheroIconoInactivo ("iconos/LamparaInactiva.xpm");
		bitmapElementoInactivo.LoadFile(ficheroIconoInactivo, TIPO_ICONOS);
		
		wxString ficheroIconoActivo ("iconos/LamparaActiva.xpm");
		bitmapElementoActivo.LoadFile(ficheroIconoActivo, TIPO_ICONOS);
	
		bitmapEnPantalla = bitmapElementoInactivo;
		
		Voltaje* v1=	new Voltaje(0.0);
		Voltaje* v2=	new Voltaje(5.0);
		/*	Por defecto. Se usan bombillas blancas. Que ahorran energa ;) */
		Luminosidad* l1=	new Luminosidad("ffffff");
		
		Luminosidad* l2=	new Luminosidad("dd1100");
		
		elementoEnvuelto = new Lampara (v1, l1, v2, l2);
		SetNombre ("Lampara");
		
		senalReposo 	= v1;
		senalActivacion	= v2;
		
}

void LamparaVisual::setElementoEnvuelto (Elemento* e)
{
	wxLogMessage ("---LamparaVisual::setElementoEnvuelto---");
	ElementoVisual::setElementoEnvuelto (e);
	Lampara* lampara = dynamic_cast<Lampara*> (e);
	senalReposo 	= lampara->getVoltajeReposo();
	senalActivacion = lampara->getVoltajeActivacion();
	SetNombre (e->getNombreElemento());
	
}

Dialogo* LamparaVisual::FabricarDialogo()
{
	return new DialogoLampara();
}

void LamparaVisual::DibujarElemento (wxDC& dc)
{
	assert (estado_actual == ACTIVO || estado_actual == INACTIVO  || 
		estado_actual == SELECCIONADO);
	int x_central = x + (ANCHURA_ICONOS/2);
	int y_central = y + (ALTURA_ICONOS/2);
	Lampara* lampara = dynamic_cast<Lampara*> (elementoEnvuelto);
		
	/* Se dibuja la lampara de la forma habitual */
	ElementoVisual::DibujarElemento (dc);
	
	/* pero si adems esta activa se rellena del color concreto */
	if (estado_actual== INACTIVO)
	{
		Luminosidad* l = lampara->getLuminosidadReposo();
		const char* color = l->toString();
		
		/* Para rellenar hay que saber el color de la lampara, la brocha
			con que se rellena el area y el color del borde del area.*/
		wxColour colorReposo = getColor (color);
		wxBrush brochaRellenado(colorReposo);
		/* Se establece la brocha activa */
		dc.SetBrush (brochaRellenado);
		
		wxColour color_borde(0,0,0);
		/* Rellena con la brocha activa el area delimitada por
			pixeles de color "colorreposo"*/
		
		dc.FloodFill (x_central, y_central, color_borde, wxFLOOD_BORDER);
	}
	
	if (estado_actual== ACTIVO)
	{
		Luminosidad* l = lampara->getLuminosidadActivacion();
		const char* color = l->toString();
		
		wxColour colorActivacion = getColor (color);
		wxBrush brochaRellenado(colorActivacion);
		dc.SetBrush (brochaRellenado);
		
		wxColour color_borde(0,0,0);
		dc.FloodFill (x_central, y_central, color_borde, wxFLOOD_BORDER);
	}
}

wxColour LamparaVisual::getColor (const char* color)
{
	int r, g, b;
	
	sscanf (color, "%02x%02x%02x", &r, &g, &b);
	
	wxColour color_a_devolver (r, g, b);
	
	return color_a_devolver;
}
