#include "temporizadorvisual.h"

TemporizadorVisual::TemporizadorVisual()
{	
	/*	El bitmap ya ha sido
		creado en el constructor de la clase padre ElementoVisual,
		por lo que aqui podemos cargarlo sin preocuparnos de reservar */
		
		wxString ficheroIcono ("iconos/Temporizador.xpm");
		bitmapEnPantalla.LoadFile (ficheroIcono, TIPO_ICONOS);
		
		wxString ficheroIconoInactivo ("iconos/RelojInactivo.xpm");
		bitmapElementoInactivo.LoadFile(ficheroIconoInactivo, TIPO_ICONOS);
		
		wxString ficheroIconoActivo ("iconos/RelojActivo.xpm");
		bitmapElementoActivo.LoadFile(ficheroIconoActivo, TIPO_ICONOS);
			

		Voltaje* voltajeEntradaInicial	=	new Voltaje(0.0);
		Voltaje* voltajeSalidaInicial	=	new Voltaje(0.0);	
		Voltaje* voltajeActivacion		=	new Voltaje(5.0);
		Voltaje* voltajeSalidaEnTimeout=	new Voltaje(5.0);
		
		/*	Los temporizadores esperan 5 intervalos de tiempo por defecto */	
		unsigned int tiempoMax= 	5;
	
		SetNombre ("Temporizador");
		elementoEnvuelto = new Temporizador (
			voltajeEntradaInicial,	voltajeSalidaInicial, 
			voltajeActivacion,		voltajeSalidaEnTimeout, tiempoMax);
		
		bitmapEnPantalla = bitmapElementoInactivo;
		
		/* Se toma nota de los valores necesarios para que el elemento
			pase a estado de reposo o de activacion */
		Temporizador* aux = dynamic_cast<Temporizador*> (elementoEnvuelto);
		senalReposo 	= aux->getVoltajeEntradaInicial();
		senalActivacion	= aux->getVoltajeActivacion();
}

void TemporizadorVisual::setElementoEnvuelto (Elemento* e)
{
	wxLogMessage ("---TemporizadorVisual::setElementoEnvuelto---");
	ElementoVisual::setElementoEnvuelto (e);
	Temporizador* temporizador = dynamic_cast<Temporizador*> (e);
	this->elementoEnvuelto = temporizador;
	senalReposo 	= temporizador->getVoltajeEntradaInicial();
	senalActivacion = temporizador->getVoltajeActivacion();
	SetNombre (e->getNombreElemento());
	
}

Dialogo* TemporizadorVisual::FabricarDialogo()
{
	return new DialogoTemporizador();
}

void TemporizadorVisual::manejarEntrada (Senal* s, int posicion)
{
	/* Se maneja la entrada de la forma habitual*/
	ElementoVisual::manejarEntrada(s, posicion);
	
//	/*	Si el nuevo estado es activacion, se lanza el temporizador
//		con un intervalo de 1000 milisegundos */
//	if (estado_actual == ACTIVO ) Start (1000);
//	
//	if (estado_actual == INACTIVO ) Stop();
}

int TemporizadorVisual::setEntrada(Senal* entrada, int posicion)
{
	assert (entrada!=NULL);
	assert (posicion>=0);
	
	/*	Es necesario reconvertir el puntero de Elemento* a Temporizador*
		ya que de lo contrario no se podria llamar a actualizar */
	Temporizador* temporizadorEnvuelto 	= dynamic_cast<Temporizador*> 
											(elementoEnvuelto);
	Voltaje* voltajePasado 				= dynamic_cast<Voltaje*>(entrada);
	
	bool hay_que_activar = temporizadorEnvuelto->esVoltajeDeActivacion
									(voltajePasado);
	if (hay_que_activar)
	{
		/*	Se lanza un evento a intervalos de mil milisegundos 
			solo si es el voltaje adecuado */
		bool detenerse_tras_intervalo=false;
		Start(1000, detenerse_tras_intervalo);
	}
	
	this->manejarEntrada (entrada, posicion);
	int codigo = elementoEnvuelto->setEntrada(entrada, posicion);
	return codigo;
	
}

void TemporizadorVisual::ConmutarEntrada()
{
	int codigo=-1;
	if (estado_actual == INACTIVO)
	{
		manejarEntrada (senalActivacion, POS_ENTRADA);
		codigo = elementoEnvuelto->setEntrada (senalActivacion);
		
	}
	else if (estado_actual == ACTIVO)
	{
		manejarEntrada (senalReposo, POS_ENTRADA);
		codigo = elementoEnvuelto->setEntrada (senalReposo);
	}
	assert (codigo!=-1);
	
}

void TemporizadorVisual::Notify()
{
	/*	Es necesario reconvertir el puntero de Elemento* a Temporizador*
		ya que de lo contrario no se podria llamar a actualizar */
	Temporizador* temporizadorEnvuelto = dynamic_cast<Temporizador*> (elementoEnvuelto);
	/*	El temporizador de esta instancia se actualiza a la vez que el de WxWidgets */
	temporizadorEnvuelto->actualizar();
	int tiempoActual=temporizadorEnvuelto->getTiempoActual();
	wxLogMessage ("El tiempo actual es:%i", tiempoActual);
	
	int tiempoMax=temporizadorEnvuelto->getTiempoMax();
	
	/* Si el tiempo que lleva contabilizado nuestra instancia ya es superior
		al tolerado, se detiene*/
	if (tiempoActual>tiempoMax) 
	{
		Stop();
		temporizadorEnvuelto->reset();
	}
	assert (paginaPadre!=NULL);
	wxPaintEvent evento_de_repintado;
	wxPostEvent (paginaPadre, evento_de_repintado);
	
}

