#include "pulsador.h"
#include "constantes.h"
#include "codificador.h"

#include <iostream>
using namespace std;

Pulsador::Pulsador(Conexion* c1, Voltaje* v1, Conexion* c2, Voltaje* v2)
{
	this->c1=c1;
	this->v1=v1;
	this->c2=c2;
	this->v2=v2;
	entradas[POS_ENTRADA]=c1;
	salidas[POS_SALIDA]=v1;
	strcpy (nombreElemento, "pulsador");	
}

Pulsador::~Pulsador()
{
//	XMLString::release (&etqElemento);
}


Voltaje* Pulsador::getVoltajeReposo()
{
	return v1;
}

Voltaje* Pulsador::getVoltajeActivacion()
{
	return v2;
}

Conexion* Pulsador::getConexionReposo()
{
	return c1;
}

Conexion* Pulsador::getConexionActivacion()
{
	return c2;
}

void		Pulsador::setConexionReposo		(Conexion* c)
{
	/*	Liberamos la conexion anterior*/
	free (c1);
	/* Y hacemos que c1 apunte a la nueva conexion */
	c1=c;
}
void		Pulsador::setConexionActivacion	(Conexion* c)
{
	/*	Liberamos la conexion anterior*/
	free (c2);
	/* Y hacemos que c1 apunte a la nueva conexion */
	c2=c;
}
void		Pulsador::setVoltajeReposo		(Voltaje* v)
{
	/*	Liberamos el voltaje anterior*/
	free (v1);
	/* Y hacemos que v1 apunte al nuevo voltaje */
	v1=v;
}
void		Pulsador::setVoltajeActivacion	(Voltaje* v)
{
	/*	Liberamos el voltaje anterior*/
	free (v2);
	/* Y hacemos que v1 apunte al nuevo voltaje */
	v2=v;

}

int Pulsador::setEntrada(Senal* s, int posicion)
{
	
	bool senal_es_nula = (s==NULL);
	assert (senal_es_nula==false);
	assert (posicion<=totalEntradas);
	assert (posicion>=0);
	
	assert (totalSuscripciones>=0);
	int codigo;
	
	/*	Si el voltaje que introducimos es el de activacion se devuelve
		por la salida correspondiente la salida */
	Conexion* c=(Conexion*) s;
	
#ifdef DEBUG
	printf ("-----------PULSADOR----------\n");
	printf ("La conexion pasiva es %b\n", c1->getValor());
	printf ("La conexion activa es %b\n", c2->getValor());
	printf ("El voltaje pasivo es %f\n", v1->getValor());
	printf ("El voltaje activo es %f\n", v2->getValor());	
	printf ("-----------PULSADOR----------\n");
#endif

	if ( c->getValor()==c2->getValor() )
	{
		salidas[POS_SALIDA]=v2;
		#ifdef DEBUG_PULSADOR
		printf ("Se activa el pulsador %s\n", nombreElemento);
		#endif
		codigo = Elemento::setEntrada((Senal*)c,posicion);
	}
	
	if ( c->getValor()==c1->getValor() )
	{
		//assert (false);
		#ifdef DEBUG_PULSADOR
		printf ("Se desactiva el pulsador %s\n", nombreElemento);
		#endif
		salidas[POS_SALIDA]=v1;
		codigo = Elemento::setEntrada((Senal*)c,posicion);
	}
	assert (codigo==ENTRADA_CAMBIADA);
	/*	La entrada no produjo efectos */
	return ENTRADA_CAMBIADA;
}


void Pulsador::construir (DOMElement* nodo)
{
	assert (nodo!=NULL);
	obtenerDatosComunes (nodo);
	
	/* Obtenemos los valores de los distintos nodos */
	c1 = (Conexion*)	getSenal ( 5, "entradareposo", nodo);
	v1 = (Voltaje*) 	getSenal ( 1, 	"salidareposo", nodo);
	c2 = (Conexion*) 	getSenal ( 5, "entradaactivacion", nodo);
	v2 = (Voltaje*) 	getSenal ( 1, 	"salidaactivacion", nodo);	
	
	entradas[POS_ENTRADA]=c1;
	salidas[POS_SALIDA]=v1;
}

NodoXML Pulsador::getXML ( DOMImplementation *impl )
{
	//etqElemento se hereda por parte de XMLSerializable	
	//etqElemento=XMLString::transcode ( "pulsador" );
	
	DOMDocument* doc= impl->createDocument ();	
	#define _debug_pulsador_construir
	#ifdef _debug_pulsador_construir
		cout << "--Pulsador::getNodo() invocado--" << endl;
	#endif
	doc = getDocumentoXMLBase ( doc, "pulsador",nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */

	#ifdef _debug_pulsador_construir
		cout << "anadiendo entrada reposo" << endl;
	#endif
	
	anadirElementoXML (doc, "entradareposo", 		getConexionReposo() 	);
	
	#ifdef _debug_pulsador_construir
		cout << "anadiendo salida reposo" << endl;
	#endif
	
	anadirElementoXML (doc, "salidareposo", 		getVoltajeReposo()		);
	
	#ifdef _debug_pulsador_construir
		cout << "anadiendo entrada activacion" << endl;
	#endif
	
	anadirElementoXML (doc, "entradaactivacion",	getConexionActivacion() );
	
	#ifdef _debug_pulsador_construir
		cout << "anadiendo salida activacion" << endl;
	#endif
	
	anadirElementoXML (doc, "salidaactivacion",		getVoltajeActivacion() 	);
	#ifdef _debug_pulsador_construir
		cout << "---Pulsador::getNodo() acabado---" << endl;
	#endif
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	DOMElement* elementoRaiz 	= doc->getDocumentElement();
	fragmento->appendChild ( elementoRaiz );
	return fragmento;
}
