#include "xmlserializable.h"
#include <iostream>

using namespace std;

char* XMLSerializable::recuperarNombreElemento (DOMElement* elemento)
{

	/* Recuperamos el nodo con la etiqueta nombreelemento */
	XMLCh* etiqueta = XMLString::transcode ( "nombreelemento" );
	DOMNodeList* lista = elemento->getElementsByTagName ( etiqueta );
	DOMNode *nodoNombre= lista->item(0);
	// Una vez recuperado el nombre hay que pasar al nodo hijo
	// que es donde est almacenado el nombre
	nodoNombre=nodoNombre->getFirstChild();
	
	const XMLCh* nombreRecuperado=nodoNombre->getNodeValue();
	char *nombreAEscribir=XMLString::transcode (nombreRecuperado);
//	cout << "El nombreelemento es:" << nombreAEscribir << endl;

	
	/*	Se libera la memoria utilizada temporalmente por la cadena */
	XMLString::release ( &etiqueta );
	
	return nombreAEscribir;
}



DOMDocument* XMLSerializable::getDocumentoXMLBase ( DOMDocument* doc,
	char* nombreElemento, char* textoNombreElemento, int pos_x, int pos_y )
{
	
//	#define __debug__
	
	//cout << "getDocumentoXMLBase llamado para " << nombreElemento << endl;
	DOMElement* elemento=doc->createElement ( C (nombreElemento) );
	doc->appendChild ( elemento );
	#ifdef __debug__
		cout << "Documento creado\n";
	#endif
	// Se aaden los atributos de posicion
	elemento->setAttribute ( C("x") , C( pos_x ) );
	elemento->setAttribute ( C("y") , C( pos_y ) );
	
	#ifdef __debug__
		cout << "Atributos de posicion puestos\n";
	#endif
	
	//Se aade la etiqueta "nombreelemento"...
	DOMElement*	elemNombreElemento 	= doc->createElement  
										( C("nombreelemento") );
	#ifdef __debug__
		cout << "Nombre del elemento creado\n";
	#endif
	//Esta etiqueta se aade al elemento...
	elemento->appendChild ( elemNombreElemento );
	
	
	#ifdef __debug__
		cout << "Nombre del elemento anadido\n";
	#endif
	
	//...y dentro de esa etiqueta se mete el texto del nombre de verdad
	DOMText*	txtElemento			= doc->createTextNode 
										( C ( textoNombreElemento ) );
	elemNombreElemento->appendChild ( txtElemento );
	#ifdef __debug__
		cout << "Contenido del elemento anadido\n";
	#endif
	
	return doc;
}


void XMLSerializable::anadirElementoXML ( DOMDocument* documento,
	char* etiqueta, Senal* senal)
{
//	#define __debug_anadir
	DOMElement* raiz = documento->getDocumentElement();
//	cout << "AnadirElementoXML para: " << etiqueta << endl;	
	DOMElement* etiquetaSenal 	= documento->createElement 	
					( C( etiqueta ) );
	#ifdef __debug_anadir
		cout << "Creada etiqueta " << etiqueta << endl;
	#endif
	
	raiz->appendChild ( etiquetaSenal );
	
	
	#ifdef __debug_anadir
		cout << "Anadida senal " << endl;
	#endif
	
	DOMText*	valorSenal		= documento->createTextNode 
									( C( senal->toString()  ) );
	#ifdef __debug_anadir
		cout << "Anadido valor etiqueta " << senal->toString() << endl;
	#endif
	etiquetaSenal->appendChild ( valorSenal );

}


Senal* XMLSerializable::getSenal(unsigned int tipoSenal, const char* nombre,
		DOMElement* elemento)
{
	
	XMLCh* etiqueta = XMLString::transcode ( nombre );
	DOMNodeList* lista = elemento->getElementsByTagName ( etiqueta );
	
	DOMNode *nodoNombre= lista->item(0);
	/* Es totalmente necesario que exista el elemento. Si no 
		es asi quiz es porque la etiqueta que nos han pedido
		buscar no est dentro de este elemento*/
	assert (nodoNombre!=NULL);
	
	
	nodoNombre=nodoNombre->getFirstChild();
	const XMLCh* contenidoEtiqueta=nodoNombre->getNodeValue();
	
	
	Senal* senalADevolver=NULL;
	/*	En funcion de la clase de seal que se desee construir */
	switch (tipoSenal)
	{
		/*	Cuidado si se cambian las equivalencias entre codigos de
			tipo de seal y valores. Estan definidas en senal.h*/
			
			/* Si se pide un voltaje (1) */
		case VOLTAJE :
			{
				char *valor = XMLString::transcode ( contenidoEtiqueta );
				senalADevolver= new Voltaje ( valor );
				XMLString::release ( &valor );
				break;
			}
			/* Si se pide una luminosidad (2) */
		case LUMINOSIDAD :
			{
				char *valor = XMLString::transcode ( contenidoEtiqueta );
				senalADevolver= new Luminosidad ( valor );
				XMLString::release ( &valor );
				
				break;
			}
			/* Si se pide una seal giro (3) */
		case GIRO :
			{
				char *valor = XMLString::transcode ( contenidoEtiqueta );
				senalADevolver= new Giro ( valor );
				XMLString::release ( &valor );
				break;
			}
			/* Si se pide una seal magnetismo (4) */
		case MAGNETISMO:
			{
				XMLCh* conectado = XMLString::transcode ( "on" );
				XMLCh* noconectado = XMLString::transcode ( "off" );
				int estaConectado = 
					XMLString::compareString ( contenidoEtiqueta, conectado );
				int noestaConectado = 
					XMLString::compareString ( contenidoEtiqueta, noconectado );
				if ( estaConectado == 0 )
				{
					senalADevolver = new Magnetismo ( true );
				}
				if ( noestaConectado == 0 )
				{
					senalADevolver = new Magnetismo ( false );
				}
				XMLString::release ( &conectado );
				XMLString::release ( &noconectado );
				break;
			}
			/* Conexion es de tipo 5*/
		case CONEXION :
			{
				XMLCh* conectado = XMLString::transcode ( "conectado" );
				XMLCh* noconectado = XMLString::transcode ( "no-conectado" );
				int estaConectado = 
					XMLString::compareString ( contenidoEtiqueta, conectado );
				int noestaConectado = 
					XMLString::compareString ( contenidoEtiqueta, noconectado );
				if ( estaConectado == 0 )
				{
					senalADevolver = new Conexion ( true );
				}
				if ( noestaConectado == 0 )
				{
					senalADevolver = new Conexion ( false );
				}
				XMLString::release ( &conectado );
				XMLString::release ( &noconectado );
				break;
			}
			
			/* Y frecuencia es de tipo 6 */
		case FRECUENCIA :
			{
				char *valor = XMLString::transcode ( contenidoEtiqueta );
				senalADevolver= new Frecuencia ( valor );
				XMLString::release ( &valor );
				break;
			}
		default:
			{
				assert (false);
				/* Es necesario que nos digan qu tipo de seal se necesita*/
			}
	} /* Fin del switch */
	return senalADevolver;
}

int XMLSerializable::getAtributo (DOMElement* elemento, char *attr)
{
	/*	Valores recuperados de los atributos XML */
	unsigned int intRecuperado;
	
	
	/*	Necesitamos codificar las cadenas en XMLCh (no sirven los char) */	
	XMLCh* cadenaAtributo;
	

	cadenaAtributo = XMLString::transcode ( attr );
	

	/* Se recuperan los atributos... */
	const XMLCh* cadenaRecuperada = elemento->getAttribute ( cadenaAtributo );


	/* Y se parsean...*/
	intRecuperado = XMLString::parseInt ( cadenaRecuperada );

	
	/*	Se liberan las cadenas */
	XMLString::release ( &cadenaAtributo );
	
	return intRecuperado;

}

int XMLSerializable::muestra (DOMNode *n, bool printOutEncounteredEles)
{
    DOMNode *child;
    int count = 0;
    if (n) {
        if (n->getNodeType() == DOMNode::ELEMENT_NODE)
		{
            if(printOutEncounteredEles) {
                char *name = XMLString::transcode(n->getNodeName());
                XERCES_STD_QUALIFIER cout <<"----------------------------------------------------------"<<XERCES_STD_QUALIFIER endl;
                XERCES_STD_QUALIFIER cout <<"Encountered Element : "<< name << XERCES_STD_QUALIFIER endl;
                
                XMLString::release(&name);
			
                if(n->hasAttributes()) {
                    // get all the attributes of the node
                    DOMNamedNodeMap *pAttributes = n->getAttributes();
                    int nSize = pAttributes->getLength();
                    XERCES_STD_QUALIFIER cout <<"\tAttributes" << XERCES_STD_QUALIFIER endl;
                    XERCES_STD_QUALIFIER cout <<"\t----------" << XERCES_STD_QUALIFIER endl;
                    for(int i=0;i<nSize;++i) {
                        DOMAttr *pAttributeNode = (DOMAttr*) pAttributes->item(i);
                        // get attribute name
                        char *name = XMLString::transcode(pAttributeNode->getName());
                        
                        XERCES_STD_QUALIFIER cout << "\t" << name << "=";
                        XMLString::release(&name);
                        
                        // get attribute type
                        name = XMLString::transcode(pAttributeNode->getValue());
                        XERCES_STD_QUALIFIER cout << name << XERCES_STD_QUALIFIER endl;
                        XMLString::release(&name);
                    }
                }
            }
			++count;
		}
        for (child = n->getFirstChild(); child != 0; child=child->getNextSibling())
            count += muestra(child, printOutEncounteredEles);
    }
    return count;
}
