#ifndef _xmlserializable_
#define _xmlserializable_

#define XERCES_TMPLSINC
//#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/validators/common/Grammar.hpp>	
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMErrorHandler.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>

#ifdef XERCES_CPP_NAMESPACE_USE
XERCES_CPP_NAMESPACE_USE
#endif	

#include "senal.h"
#include "codificador.h"
#include "nodoxml.h"
/* 	Esta clase abstracta proporciona el interfaz necesario para que un elemento
	pueda ser ledo desde XML y escrito hacia un fichero XML */
class XMLSerializable {
	
	public:
		/*	Esta funcion devuelve el nodo DOM que representa a un elemento*/
		virtual NodoXML getXML ( DOMImplementation* impl ) =0 ;
	
		/* 	Esta funcion rellena los valores del elemento con
			los que se encuentren en un nodo XML */
		virtual void construir (DOMElement*) = 0 ;
		
		int muestra (DOMNode *n, bool printOutEncounteredEles);
	protected:
		/*	Esta funcion recupera el nombre del elemento de un 
			nodo XML y lo pone como nombre de este elemento */
		char* recuperarNombreElemento (DOMElement* elemento);
		
		/*	Este metodo construye un documento XML base que representa
			las principales caracteristicas del elemento (como nombre o
			posicion) y deja a los metodos de clases hija la tarea de
			rellenar las etiquetas XML que describen el resto de
			caractersticas. Recibe un documento vacio y devuelve
			uno parcialmente rellenado */
		DOMDocument* getDocumentoXMLBase ( DOMDocument* doc,
											char* nombreElemento,
											char* textoNombreElemento,
											int pos_x, int pos_y );
		
		/*	Mtodo auxiliar: Permite crear un pequeo elemento XML que
			representa una determinada seal de voltaje, de giro...*/
		void anadirElementoXML ( DOMDocument* documento,
			char* etiqueta, Senal* senal);		
		
		/*	Esta funcion construye una seal del tipo indicado a partir
		del codigo que seal que nos pidan y del nombre.
		Implementa el patron Factory Method, para poder crear seales
		de cualquier tipo en momentos posteriores.*/
		Senal* getSenal(unsigned int codigo, const char* nombre,
					DOMElement* elemento);
		
		/*	Este metodo obtiene los atributos que marcan la posicion de
			un elemento */
		int getAtributo (DOMElement* elemento, char *attr);
		
		
		/*	Aqui se construira el XML que representa un elemento. El metodo
			getNodo lo rellenara y lo escribira en el fichero indicado */
		NodoXML	nodo;
		
		/*	El constructor de un elemento debe indicar el nombre de la 
			etiqueta XML que debe llevar un elemento (por ejemplo
			"pulsador" para poder hacer <pulsador>...</pulsador>*/
		XMLCh*	etqElemento;
	private:
	
};


#endif
